function getCustomProperties(formaParam, sendParam) {
    Debug.console('************** 入力チェックプログラム：カスタムプロパティを設定 ****************');

    return {
        error: false,
        data: {}
    };
}

function validate(formaParam, sendParam, resultObj) {
    Debug.console('************** 入力チェックプログラム：ユーザバリデーション実行 ****************');

    // 単項目は通常のアイテム同様、sendParam.%フィールド識別ID%に存在します。
    let applyUserName = sendParam.apply_user_name;
    let department = sendParam.department;
    let applyDate = sendParam.apply_date;
    let address = sendParam.address;
    let closestStation = sendParam.closest_station;

    // 必須チェック
    let isInputedApplyDate = required(applyDate, resultObj, 'Application date', 'apply_date');
    let isInputedApplyUserName = required(applyUserName, resultObj, 'Applicant', 'apply_user_name');
    let isInputedDepartment = required(department, resultObj, 'Affiliation department', 'department');
    required(address, resultObj, 'Applicant\'s address', 'address');
    required(closestStation, resultObj, 'Closest station', 'closest_station');

    // 桁数チェック
    maxLength(applyUserName, resultObj, 'Applicant', 'apply_user_name', 250);
    maxLength(department, resultObj, 'Affiliation department', 'department', 250);
    maxLength(address, resultObj, 'Applicant\'s address', 'address', 250);

    // 有効な日付に変換できるミリ秒かチェック
    if (isInputedApplyDate) {
        let applyDateConverted = new Date(Number(applyDate));
        if (isNaN(applyDateConverted.getTime())) {
            ImFormaUtil.addValidationError(resultObj, '"Application date" is not a valid date.', 'valid_apply_date', ['apply_date']);
        } else {
            // 範囲チェック
            if (applyDateConverted < new Date('2018/01/01') || applyDateConverted >= new Date('2020/12/31')) {
                ImFormaUtil.addValidationError(resultObj, 'Please input "Application date" within the range of 2018/01/01 to 2020/12/31.', 'range_apply_date', ['apply_date']);
            }
        }
    }

    // ユーザがシステムに存在するかチェックします。
    if (isInputedApplyUserName) {
        let userCondition = new AppCmnSearchCondition();
        userCondition.addCondition('user_name', applyUserName);
        let userManager = new IMMUserManager();
        let userResult = userManager.countUser(userCondition, new Date, 'en');
        if (userResult.data === 0) {
            ImFormaUtil.addValidationError(resultObj, 'Employee name which does not exist in "Applicant" is entered.', 'exists_apply_user_name', ['apply_user_name']);
        }
    }

    // 所属部署がシステムに存在するかチェックします。
    if (isInputedDepartment) {
        let deptCondition = new AppCmnSearchCondition();
        deptCondition.addCondition('department_name', department);
        let companyManager = new IMMCompanyManager();
        let deptResult = companyManager.countDepartment(deptCondition, new Date(), 'en');
        if (deptResult.data === 0) {
            ImFormaUtil.addValidationError(resultObj, 'The department name which does not exist in "Affiliation department" is entered.', 'exists_department', ['department']);
        }
    }

    // テーブル定義は明細テーブル、グリッドテーブル同様、sendParam.%テーブル識別ID% に配列データとして存在します。
    let routeList = sendParam.route_list;

    for (let i = 0; i < routeList.length; i++) {
        let route = routeList[i];

        // 入力値が数値（文字列）であるかチェック
        if (!isBlank(route.three_months)) {
            isNumber(route.three_months, resultObj, '3 months(commuter pass)', 'three_months', i);    
        }
        if (!isBlank(route.one_month)) {
            isNumber(route.one_month, resultObj, '1 months(commuter pass)', 'one_month', i);
        }
        if (!isBlank(route.one_day)) {
            isNumber(route.one_day, resultObj, 'one day(round trip)', 'one_day', i);
        }

        // 1カ月分の定期代が30000円以上でないかチェックします。
        if (route.one_month >= 30000) {
            ImFormaUtil.addValidationError(resultObj, '1 month commuter pass of 30,000 yen or more is not permitted by the regulations.(line ' + (i + 1) + ')', 'limit_one_month', ['one_month'], i);
        }

        // 1レコードのいずれかの項目に入力があった場合、全ての項目を必須とします。
        let requiredList = [];
        if (!route.transportation) {
            requiredList.push('transportation');
        }
        if (!route.from) {
            requiredList.push('from');
        }
        if (!route.to) {
            requiredList.push('to');
        }
        if (!route.three_months) {
            requiredList.push('three_months');
        }
        if (!route.one_month) {
            requiredList.push('one_month');
        }
        if (!route.one_day) {
            requiredList.push('one_day');
        }
        if (requiredList.length > 0 && requiredList.length !== 6) {
            ImFormaUtil.addValidationError(resultObj, 'Please enter all items in the record.' + '(line ' + (i + 1) + ')', 'enter_all_route_record', requiredList, i);
        }
    }

    
    // 返却オブジェクトを返却
    return resultObj;
}

function required(value, resultObj, label, inputId) {
    if (isBlank(value)) {
        ImFormaUtil.addValidationError(resultObj, label + ' is required.', 'required_' + inputId, [inputId]);
        return false;
    }
    return true;
}

function maxLength(value, resultObj, label, inputId, limit) {
    if (value.length > limit) {
        ImFormaUtil.addValidationError(resultObj, 'Up to ' + limit + ' digits can be entererd for ' + label + '.', 'max_length_' + inputId, [inputId]);
        return false;
    }
    return true;
}

function isNumber(value, resultObj, label, inputId, i) {
    if (isNaN(Number(value))) {
        ImFormaUtil.addValidationError(resultObj, label + ' is only allowd to enter numerical values.' + '(line ' + (i + 1) + ')', 'number_' + inputId, [inputId]);
        return false;
    }
    return true;
}

function getErrorDisplayOrder(formaParam) {
    Debug.console('************** 入力チェックプログラム：エラーメッセージ表示順を指定 ****************');

    var arry = [
        'range_apply_date',
        'exists_department',
        'required_address'
    ];

    return arry;
}
